import java.util.*;

int MAX_VISIBLE = 10;
int BOTTOM_MARGIN = 100;
int TOP_MARGIN = 50;

ArrayList transactions = null; //global List of the data
ArrayList subjectEntries;

int curIdx = 0;
int maxCnt = 0;
int curKeyIdx = (int) 'a';

void setup()
{
  size(600,400, P2D);
  frame.setResizable(true);
  background(0);
  frameRate(1000); //that is, as fast as possible

  textFont(loadFont("GillSans-24.vlw"), 16); 

  long before = millis();
  this.transactions = loadTransactions();

  println("loaded " + this.transactions.size() + " transactions in " + (millis() - before) + " miiliseconds.");

  subjectEntries = TransactionUtils.countUniqueSubjects(this.transactions);
  TransactionUtils.sortUniqueSubjectsByCount(subjectEntries, TransactionUtils.DESCENDING);

  maxCnt = TransactionUtils.getIntValueForEntry(subjectEntries.get(0));
 
  printUniqueSubjects(10);
}

void printUniqueSubjects(int howMany)
{
  for (int i = 0 ; i < howMany; i++)
  {
    Map.Entry entry = (Map.Entry) (subjectEntries.get(i));
    println("entry = " + entry.getKey() + " : " + entry.getValue());  
  }   
}

void draw()
{
  background(0);

  float xinc = ((float)width / (float)MAX_VISIBLE);
  for (int i = curIdx; i < curIdx + MAX_VISIBLE; i++)
  {   
    int stPos = i - curIdx;
    Map.Entry me = (Map.Entry) subjectEntries.get(i);
    int val = TransactionUtils.getIntValueForEntry(me);
    String key = TransactionUtils.getStringKeyForEntry(me);
    
    fill(128);
    
    float logAll = (float)Math.log(maxCnt) + 1f;
    float logVal = (float)Math.log(val) + 1f;
    
    rect( (xinc * stPos) , height - BOTTOM_MARGIN, xinc, - ((height-TOP_MARGIN-BOTTOM_MARGIN) * (logVal / logAll )));   
    textAlign(CENTER);
    text("" + val, (xinc * stPos) + (xinc / 2f), (height - BOTTOM_MARGIN) - ((height-TOP_MARGIN-BOTTOM_MARGIN) * (logVal / logAll)) );
    
    textAlign(LEFT);
    
    fill(255,0,0,255);
    pushMatrix();
    translate((xinc * stPos) + (xinc * .2f), height - BOTTOM_MARGIN + 10);
    rotate(.24);    
    text(key, 0, 0);
    popMatrix();
    
  }

  textAlign(RIGHT);
  text("August 2008 - Total Number of Items per Unique Subject", width - 15, 20); 
}

void keyPressed()
{
  println("key pressed = " + key + ", ascii = " + (int) key); 
  
  if (key == 'a')
  {
    if (curIdx < subjectEntries.size() - MAX_VISIBLE)
    {
      curIdx++;
    }
  }
  else if ( key == 'z')
  {
    if (curIdx > 0)
    {
      curIdx--;
    }
  }
  
  if (key == 's')
  {
    if (curIdx < subjectEntries.size() - (subjectEntries.size() / 50) - (MAX_VISIBLE))
    {
      curIdx += subjectEntries.size() / 50;
    }
    else
    {
       curIdx = subjectEntries.size() - MAX_VISIBLE;
    }
  }
  else if ( key == 'x')
  {
    if (curIdx > subjectEntries.size() / 50)
    {
      curIdx -= subjectEntries.size() / 50;
    }
    else
    {
       curIdx = 0; 
    }
  }
  else if (key == '1')
  {
   TransactionUtils.sortUniqueSubjectsByCount(subjectEntries, TransactionUtils.ASCENDING);
  }
  else if (key == '2')
  {
   TransactionUtils.sortUniqueSubjectsByCount(subjectEntries, TransactionUtils.DESCENDING);
  }  
}

public ArrayList loadTransactions()
{
  ArrayList transactions = new ArrayList();

  File[] fileArr = listFiles(dataPath(""));

  for (int fn = 0; fn < fileArr.length; fn++) 
  {
    File f = (File) fileArr[fn];    

    if (!f.getName().endsWith(".txt"))
    {
      continue;
    }

    println("loading file... \"" + f.getName() + "\"");
    String[] lines = loadStrings(f);

    for (int i = 0; i < lines.length; i++)
    {
      transactions.add(parseTransaction(lines[i])); 
    }
  }

  return transactions;
}


// This function returns all the files in a directory as an array of File objects
// This is useful if you want more info about the file
public File[] listFiles(String dir) {
  File file = new File(dir);
  if (file.isDirectory()) {
    File[] files = file.listFiles();
    return files;
  } 
  else {
    // If it's not a directory
    return null;
  }
}

//create a new transaction obejct from a comma separated line. For this example we are only storing subjects.
public Transaction parseTransaction(String line)
{
  String sections[] = split(line, ",");

  Transaction t = new Transaction();
  
  int numSubjects = sections.length;
  t.subjects = new String[numSubjects];
  arraycopy(sections, 0, t.subjects, 0, numSubjects);

  return t;
}








